@@ -68,6 +68,8 @@ function setup_defaults
 
   common_defaults
   GLOBALTIMER=$("${AWK}" 'BEGIN {srand(); print srand()}')
+  # shellcheck disable=SC2034
+  ISODATESTART=$(date +"%Y-%m-%dT%H:%M:%SZ")
 
   set_yetus_version
 
@@ -93,9 +95,10 @@ function setup_defaults
 
   CONTINUOUS_IMPROVEMENT=false
 
-  PROC_LIMIT=1000
+  PROC_LIMIT=2000
   REEXECED=false
   RESETREPO=false
+  REPORT_UNKNOWN_OPTIONS=true
   BUILDMODE=${BUILDMODE:-patch}
   # shellcheck disable=SC2034
   BUILDMODEMSG=${BUILDMODEMSG:-"The patch"}
@@ -200,22 +203,24 @@ function add_header_line
 ## @replaceable  no
 ## @param        +1/0/-1/H
 ## @param        subsystem
+## @param        logfile
 ## @param        string
-function add_vote_table
+function add_vote_table_v2
 {
   declare value=$1
   declare subsystem=$2
-  shift 2
+  declare logfile=$3
+  shift 3
 
   # apparently shellcheck doesn't know about declare -r
   #shellcheck disable=SC2155
   declare -r elapsed=$(stop_clock)
   declare filt
 
-  yetus_debug "add_vote_table ${value} ${subsystem} ${elapsed} ${*}"
+  yetus_debug "add_vote_table_v2 >${value}< >${subsystem}< >${logfile}< >${elapsed}< ${*}"
 
   if [[ "${value}" = H ]]; then
-    TP_VOTE_TABLE[${TP_VOTE_COUNTER}]="|${value}| | | ${subsystem} |"
+    TP_VOTE_TABLE[${TP_VOTE_COUNTER}]="|${value}| | | | ${subsystem} |"
     ((TP_VOTE_COUNTER=TP_VOTE_COUNTER+1))
     return
   fi
@@ -231,7 +236,7 @@ function add_vote_table
   done
 
   # shellcheck disable=SC2034
-  TP_VOTE_TABLE[${TP_VOTE_COUNTER}]="| ${value} | ${subsystem} | ${elapsed} | $* |"
+  TP_VOTE_TABLE[${TP_VOTE_COUNTER}]="| ${value} | ${subsystem} | ${elapsed} | ${logfile} | $* |"
   ((TP_VOTE_COUNTER=TP_VOTE_COUNTER+1))
 
   if [[ "${value}" = -1 ]]; then
@@ -239,6 +244,18 @@ function add_vote_table
   fi
 }
 
+## @description  Deprecated. Use add_vote_table_v2 instead.
+## @audience     public
+## @stability    stable
+## @replaceable  no
+function add_vote_table
+{
+  declare param1=$1
+  declare param2=$2
+  shift 2
+  add_vote_table_v2 "${param1}" "${param2}" "" "$@"
+}
+
 ## @description  Report the JVM vendor and version of the given directory
 ## @stability    stable
 ## @audience     private
@@ -301,9 +318,7 @@ function prepopulate_footer
     add_footer_table "Personality" "${PERSONALITY}"
   fi
 
-  gitrev=$("${GIT}" rev-parse --verify --short HEAD)
-
-  add_footer_table "git revision" "${PATCH_BRANCH} / ${gitrev}"
+  add_footer_table "git revision" "${PATCH_BRANCH} / ${GIT_BRANCH_SHA}"
 }
 
 ## @description  Last minute entries on the footer table
@@ -387,6 +402,16 @@ function big_console_header
 {
   local text="$*"
   local spacing=$(( (75+${#text}) /2 ))
+
+  if [[ "${ROBOTTYPE}" == 'githubactions' ]]; then
+    echo "::endgroup::"
+    echo "::group::${text}"
+  fi
+
+  if [[ "${ROBOTTYPE}" == 'buildkite' ]]; then
+    echo "--- ${text}"
+  fi
+
   printf '\n\n'
   echo "============================================================================"
   echo "============================================================================"
@@ -538,65 +563,10 @@ function compute_gitdiff
     touch "${GITDIFFCONTENT}"
   fi
 
-  if [[ -s "${GITDIFFLINES}" ]]; then
-    compute_unidiff
-  fi
-
   popd >/dev/null || return 1
 }
 
-## @description generate an index of unified diff lines vs. modified/added lines
-## @description ${GITDIFFLINES} must exist.
-## @audience    private
-## @stability   stable
-## @replaceable no
-function compute_unidiff
-{
-  declare fn
-  declare filen
-  declare tmpfile="${PATCH_DIR}/tmp.$$.${RANDOM}"
 
-  # now that we know what lines are where, we can deal
-  # with github's pain-in-the-butt API. It requires
-  # that the client provides the line number of the
-  # unified diff on a per file basis.
-
-  # First, build a per-file unified diff, pulling
-  # out the 'extra' lines, grabbing the adds with
-  # the line number in the diff file along the way,
-  # finally rewriting the line so that it is in
-  # 'filename:diff line:content' format
-
-  for fn in "${CHANGED_FILES[@]}"; do
-    if [[ -f "${filen}" ]]; then
-      "${GIT}" diff "${filen}" \
-        | tail -n +6 \
-        | "${GREP}" -n '^+' \
-        | "${GREP}" -vE '^[0-9]*:\+\+\+' \
-        | "${SED}" -e 's,^\([0-9]*:\)\+,\1,g' \
-          -e "s,^,${filen}:,g" \
-              >>  "${tmpfile}"
-    fi
-  done
-
-  # if every file that got changed was excluded, there
-  # is no data to calculate.
-
-  if [[ -s "${tmpfile}" ]]; then
-
-    # at this point, tmpfile should be in the same format
-    # as gitdiffcontent, just with different line numbers.
-    # let's do a merge (using gitdifflines because it's easier)
-
-    # filename:real number:diff number
-    # shellcheck disable=SC2016
-    paste -d: "${GITDIFFLINES}" "${tmpfile}" \
-      | "${AWK}" -F: '{print $1":"$2":"$5":"$6}' \
-      >> "${GITUNIDIFFLINES}"
-
-    rm "${tmpfile}"
-  fi
-}
 
 
 ## @description  Print the command to be executing to the screen. Then
@@ -681,14 +651,13 @@ function yetus_usage
   echo ""
   yetus_add_option "--archive-list=<list>" "Comma delimited list of pattern matching notations to copy to patch-dir"
   yetus_add_option "--basedir=<dir>" "The directory to apply the patch to (default: current directory)"
+  yetus_add_option "--branch-default=<ref>" "If the branch isn't forced and we don't detect one in the patch name, use this branch (default 'main')"
   yetus_add_option "--branch=<ref>" "Forcibly set the branch"
-  yetus_add_option "--branch-default=<ref>" "If the branch isn't forced and we don't detect one in the patch name, use this branch (default 'master')"
-  yetus_add_option "--build-native=<bool>" "If true, then build native components (default 'true')"
-  # shellcheck disable=SC2153
-  yetus_add_option "--build-tool=<tool>" "Pick which build tool to focus around (default: autodetect from '${buildtools}')"
   yetus_add_option "--bugcomments=<bug>" "Only write comments to the screen and this comma delimited list (default: '${bugsys}')"
-  yetus_add_option "--contrib-guide=<url>" "URL to point new users towards project conventions. (default: ${PATCH_NAMING_RULE} )"
+  yetus_add_option "--build-native=<bool>" "If true, then build native components (default 'true')"
+  yetus_add_option "--build-tool=<tool>" "Pick which build tool to focus around (default: autodetect from '${buildtools}')"
   yetus_add_option "--continuous-improvement=<bool>" "If true, then do not exit with failure on branches (default: ${CONTINUOUS_IMPROVEMENT})"
+  yetus_add_option "--contrib-guide=<url>" "URL to point new users towards project conventions. (default: ${PATCH_NAMING_RULE} )"
   yetus_add_option "--debug" "If set, then output some extra stuff to stderr"
   yetus_add_option "--dirty-workspace" "Allow the local git workspace to have uncommitted changes"
   yetus_add_option "--empty-patch" "Create a summary of the current source tree"
@@ -699,25 +668,26 @@ function yetus_usage
   yetus_add_option "--java-home=<path>" "Set JAVA_HOME (In Docker mode, this should be local to the image)"
   yetus_add_option "--linecomments=<bug>" "Only write line comments to this comma delimited list (default: same as --bugcomments)"
   yetus_add_option "--list-plugins" "List all installed plug-ins and then exit"
+  yetus_add_option "--modulelist=<list>" "Specify additional modules to test (comma delimited)"
   yetus_add_option "--multijdkdirs=<paths>" "Comma delimited lists of JDK paths to use for multi-JDK tests"
   yetus_add_option "--multijdktests=<list>" "Comma delimited tests to use when multijdkdirs is used. (default: '${jdktlist}')"
-  yetus_add_option "--modulelist=<list>" "Specify additional modules to test (comma delimited)"
   yetus_add_option "--offline" "Avoid connecting to the network"
   yetus_add_option "--patch-dir=<dir>" "The directory for working and output files (default '/tmp/test-patch-${PROJECT_NAME}/pid')"
   yetus_add_option "--personality=<file>" "The personality file to load"
+  yetus_add_option "--plugins=<list>" "Specify which plug-ins to add/delete (comma delimited; use 'all' for all found) e.g. --plugins=all,-ant,-scalac (all plugins except ant and scalac)"
   yetus_add_option "--proclimit=<num>" "Limit on the number of processes (default: ${PROC_LIMIT})"
   yetus_add_option "--project=<name>" "The short name for project currently using test-patch (default 'yetus')"
-  yetus_add_option "--plugins=<list>" "Specify which plug-ins to add/delete (comma delimited; use 'all' for all found) e.g. --plugins=all,-ant,-scalac (all plugins except ant and scalac)"
+  yetus_add_option "--report-unknown-options=<bool>" "Print a warning in the report if --ignore-unknown-options=true and unknown options were found (default: ${REPORT_UNKNOWN_OPTIONS})"
   yetus_add_option "--resetrepo" "Forcibly clean the repo"
   yetus_add_option "--run-tests" "Run all relevant tests below the base directory"
   yetus_add_option "--skip-dirs=<list>" "Skip following directories for module finding"
-  yetus_add_option "--skip-system-plugins" "Do not load plugins from ${BINDIR}/test-patch.d"
+  yetus_add_option "--skip-system-plugins" "Do not load plugins from ${BINDIR}/plugins.d"
   yetus_add_option "--summarize=<bool>" "Allow tests to summarize results"
   yetus_add_option "--test-parallel=<bool>" "Run multiple tests in parallel (default false in developer mode, true in Jenkins mode)"
   yetus_add_option "--test-threads=<int>" "Number of tests to run in parallel (default defined in ${PROJECT_NAME} build)"
-  yetus_add_option "--unit-test-filter-file=<file>" "The unit test filter file to load"
   yetus_add_option "--tests-filter=<list>" "Lists of tests to turn failures into warnings"
-  yetus_add_option "--user-plugins=<dir>" "A directory of user provided plugins. see test-patch.d for examples (default empty)"
+  yetus_add_option "--unit-test-filter-file=<file>" "The unit test filter file to load"
+  yetus_add_option "--user-plugins=<dir>" "A directory of user provided plugins. (default: ${USER_PLUGIN_DIR})"
   yetus_add_option "--version" "Print release version information and exit"
 
   yetus_generic_columnprinter "${YETUS_OPTION_USAGE[@]}"
@@ -739,9 +709,9 @@ function yetus_usage
 
   echo ""
   echo "Automation options:"
-  yetus_add_option "--build-url=<url>" "Set the build location web page (Default: '${BUILD_URL}')"
-  yetus_add_option "--build-url-console=<location>" "Location relative to --build-url of the console (Default: '${BUILD_URL_CONSOLE}')"
   yetus_add_option "--build-url-artifacts=<location>" "Location relative to --build-url of the --patch-dir (Default: '${BUILD_URL_ARTIFACTS}')"
+  yetus_add_option "--build-url-console=<location>" "Location relative to --build-url of the console (Default: '${BUILD_URL_CONSOLE}')"
+  yetus_add_option "--build-url=<url>" "Set the build location web page (Default: '${BUILD_URL}')"
   yetus_add_option "--console-report-file=<file>" "Save the final console-based report to a file in addition to the screen"
   yetus_add_option "--console-urls" "Use the build URL instead of path on the console report"
   yetus_add_option "--instance=<string>" "Parallel execution identifier string"
@@ -790,6 +760,8 @@ function parse_args
 
   common_args "$@"
 
+  check_basedir_repo
+
   for i in "$@"; do
     case ${i} in
       --archive-list=*)
@@ -916,6 +888,8 @@ function parse_args
       --sentinel)
         delete_parameter "${i}"
         # shellcheck disable=SC2034
+        ROBOT=true
+        # shellcheck disable=SC2034
         SENTINEL=true
         yetus_add_array_element EXEC_MODES Sentinel
       ;;
@@ -996,6 +970,10 @@ function parse_args
     yetus_add_array_element EXEC_MODES Robot
   fi
 
+  if yetus_is_container; then
+    yetus_add_array_element EXEC_MODES InContainer
+  fi
+
   if [[ -n $UNIT_TEST_FILTER_FILE ]]; then
     if [[ -f $UNIT_TEST_FILTER_FILE ]]; then
       UNIT_TEST_FILTER_FILE=$(yetus_abs "${UNIT_TEST_FILTER_FILE}")
@@ -1017,22 +995,14 @@ function parse_args
     else
       yetus_add_array_element EXEC_MODES Docker
     fi
-    add_vote_table 0 reexec "Docker mode activated."
+    add_vote_table_v2 0 reexec "" "Docker mode activated."
     start_clock
   elif [[ "${REEXECED}" = true ]]; then
     yetus_add_array_element EXEC_MODES Re-exec
-    add_vote_table 0 reexec "Precommit patch detected."
+    add_vote_table_v2 0 reexec "" "Precommit patch detected."
     start_clock
   fi
 
-  # we need absolute dir for ${BASEDIR}
-  cd "${STARTINGDIR}" || cleanup_and_exit 1
-  BASEDIR=$(yetus_abs "${BASEDIR}")
-
-  if [[ -n ${USER_PATCH_DIR} ]]; then
-    PATCH_DIR="${USER_PATCH_DIR}"
-  fi
-
   # we need absolute dir for PATCH_DIR
   cd "${STARTINGDIR}" || cleanup_and_exit 1
   if [[ ! -d ${PATCH_DIR} ]]; then
@@ -1046,17 +1016,14 @@ function parse_args
   PATCH_DIR=$(yetus_abs "${PATCH_DIR}")
   COPROC_LOGFILE="${PATCH_DIR}/coprocessors.txt"
 
-  if [[ -n "${EXCLUDE_PATHS_FILE}" ]]; then
-    if [[ -f "${EXCLUDE_PATHS_FILE}" ]]; then
-      EXCLUDE_PATHS_FILE=$(yetus_abs "${EXCLUDE_PATHS_FILE}")
-    elif [[ -f "${BASEDIR}/${EXCLUDE_PATHS_FILE}" ]]; then
-      EXCLUDE_PATHS_FILE=$(yetus_abs "${BASEDIR}/${EXCLUDE_PATHS_FILE}")
-    elif [[ -f "${PATCH_DIR}/precommit/excluded.txt" ]]; then
-       EXCLUDE_PATHS_FILE="${PATCH_DIR}/precommit/excluded.txt"
-    else
-      yetus_error "ERROR: Excluded paths file (${EXCLUDE_PATHS_FILE}}) does not exist!"
-      cleanup_and_exit 1
-    fi
+  if [[ -f "${EXCLUDE_PATHS_FILE}" ]]; then
+    EXCLUDE_PATHS_FILE=$(yetus_abs "${EXCLUDE_PATHS_FILE}")
+  elif [[ -f "${BASEDIR}/${EXCLUDE_PATHS_FILE}" ]]; then
+    EXCLUDE_PATHS_FILE=$(yetus_abs "${BASEDIR}/${EXCLUDE_PATHS_FILE}")
+  fi
+
+  if [[ -f "${EXCLUDE_PATHS_FILE}" ]]; then
+    EXCLUDE_PATHS_FILE=$(yetus_relative_dir "${BASEDIR}" "${EXCLUDE_PATHS_FILE}")
   fi
 
   # we need absolute dir for ${CONSOLE_REPORT_FILE}
@@ -1084,7 +1051,6 @@ function parse_args
 
   GITDIFFLINES="${PATCH_DIR}/gitdifflines.txt"
   GITDIFFCONTENT="${PATCH_DIR}/gitdiffcontent.txt"
-  GITUNIDIFFLINES="${PATCH_DIR}/gitdiffunilines.txt"
 
   if [[ "${REEXECED}" = true
      && -f "${PATCH_DIR}/precommit/personality/provided.sh" ]]; then
@@ -1199,6 +1165,9 @@ function git_checkout
 
   big_console_header "Confirming git environment"
 
+  # if this still hasn't been set by now, set it.
+  PATCH_BRANCH_DEFAULT=${PATCH_BRANCH_DEFAULT:="main"}
+
   git_requires_creds
 
   if [[ ${ROBOT} == true ]]; then
@@ -1208,10 +1177,6 @@ function git_checkout
   fi
 
   cd "${BASEDIR}" || cleanup_and_exit 1
-  if [[ ! -e .git ]]; then
-    yetus_error "ERROR: ${BASEDIR} is not a git repo."
-    cleanup_and_exit 1
-  fi
 
   if [[ ${RESETREPO} == "true" ]] ; then
 
@@ -1252,7 +1217,7 @@ function git_checkout
     # we need to explicitly fetch in case the
     # git ref hasn't been brought in tree yet
     if [[ ${GIT_OFFLINE} == false ]]; then
-      if ! "${GIT}" pull --rebase; then
+      if ! "${GIT}" pull --rebase --tags --force; then
           yetus_error "ERROR: git pull is failing"
           cleanup_and_exit 1
       fi
@@ -1274,14 +1239,19 @@ function git_checkout
       # protected by OFFLINE == false
 
       if [[ "${GIT_OFFLINE}" == false ]]; then
-        if ! "${GIT}" fetch; then
-          yetus_error "ERROR: git fetch is failing"
-          cleanup_and_exit 1
-        fi
 
-        if ! "${GIT}" reset --hard FETCH_HEAD; then
-          yetus_error "ERROR: git reset is failing"
-          cleanup_and_exit 1
+        # if it is a tag, then the pull rebase should have done
+        # the trick already
+        if [[ ! -f ".git/refs/tags/${PATCH_BRANCH}" ]]; then
+          if ! "${GIT}" fetch; then
+            yetus_error "ERROR: git fetch is failing"
+            cleanup_and_exit 1
+          fi
+
+          if ! "${GIT}" reset --hard FETCH_HEAD; then
+            yetus_error "ERROR: git reset is failing"
+            cleanup_and_exit 1
+          fi
         fi
       fi
 
@@ -1313,11 +1283,15 @@ function git_checkout
     fi
   fi
 
+  if [[ -z "${GIT_BRANCH_SHA}" ]]; then
+    GIT_BRANCH_SHA=$("${GIT}" log -1 --format="%H")
+  fi
+
   return 0
 }
 
 ## @description  Confirm the given branch is a git reference
-## @descriptoin  or a valid gitXYZ commit hash
+## @description  or a valid gitXYZ commit hash
 ## @audience     private
 ## @stability    evolving
 ## @replaceable  no
@@ -1437,11 +1411,11 @@ function determine_needed_tests
   big_console_header "Determining needed tests"
   echo "(Depending upon input size and number of plug-ins, this may take a while)"
 
-  exclude_paths_from_changed_files
+  exclude_paths_from_changed_files "branch"
 
   for i in "${CHANGED_FILES[@]}"; do
     yetus_debug "Determining needed tests for ${i}"
-    personality_file_tests "${i}"
+    personality_file_tests_wrapper "${i}"
 
     for plugin in "${TESTTYPES[@]}" ${BUILDTOOL}; do
       if declare -f "${plugin}_filefilter" >/dev/null 2>&1; then
@@ -1463,8 +1437,10 @@ function apply_patch_file
 {
 
   if [[ "${INPUT_APPLIED_FILE}" ==  "${INPUT_DIFF_FILE}" ]]; then
-    BUGLINECOMMENTS=""
-    add_vote_table '-0' patch "Used diff version of patch file. Binary files and potentially other changes not applied. Please rebase and squash commits if necessary."
+    add_vote_table_v2 '-0' patch "" "Used diff version of patch file. Binary files and potentially other changes not applied. Please rebase and squash commits if necessary."
+    if [[ "${GITHUB_ACTIONS}" == true ]]; then
+      echo "::warning::Used diff version. Binary files and potentially other changes not applied. Try a rebase and/or squashing commits."
+    fi
     big_console_header "Applying diff to ${PATCH_BRANCH}"
   else
     big_console_header "Applying patch to ${PATCH_BRANCH}"
@@ -1473,7 +1449,10 @@ function apply_patch_file
   if ! patchfile_apply_driver "${INPUT_APPLIED_FILE}"; then
     echo "PATCH APPLICATION FAILED"
     ((RESULT = RESULT + 1))
-    add_vote_table -1 patch "${PATCH_OR_ISSUE} does not apply to ${PATCH_BRANCH}. Rebase required? Wrong Branch? See ${PATCH_NAMING_RULE} for help."
+    add_vote_table_v2 -1 patch "" "${PATCH_OR_ISSUE} does not apply to ${PATCH_BRANCH}. Rebase required? Wrong Branch? See ${PATCH_NAMING_RULE} for help."
+    if [[ "${GITHUB_ACTIONS}" == true ]]; then
+      echo "::error::Cannot apply PR on top of ${PATCH_BRANCH}. Verify the branch, try a rebase, and/or squashing commits."
+    fi
     bugsystem_finalreport 1
     cleanup_and_exit 1
   fi
@@ -1521,11 +1500,6 @@ function copytpbits
   # Set to be relative to ${PATCH_DIR}/precommit
   USER_PLUGIN_DIR="${PATCH_DIR}/precommit/user-plugins"
 
-  if [[ -n ${EXCLUDE_PATHS_FILE}
-    && -f ${EXCLUDE_PATHS_FILE} ]]; then
-    yetus_debug "copying '${EXCLUDE_PATHS_FILE}' over to '${PATCH_DIR}/precommit/excluded.txt'"
-    cp -pr "${EXCLUDE_PATHS_FILE}" "${PATCH_DIR}/precommit/excluded.txt"
-  fi
   if [[ -n ${PERSONALITY}
     && -f ${PERSONALITY} ]]; then
     yetus_debug "copying '${PERSONALITY}' over to '${PATCH_DIR}/precommit/personality/provided.sh'"
@@ -1619,7 +1593,7 @@ function check_reexec
     if [[ ${RESETREPO} == false ]]; then
       ((RESULT = RESULT + 1))
       yetus_debug "can't destructively change the working directory. run with '--resetrepo' please. :("
-      add_vote_table -1 precommit "Couldn't test precommit changes because we aren't configured to destructively change the working directory."
+      add_vote_table_v2 -1 precommit "" "Couldn't test precommit changes because we aren't configured to destructively change the working directory."
     else
 
       apply_patch_file
@@ -1814,10 +1788,22 @@ function modules_messages
         echo "${MODULE_STATUS_MSG[${modindex}]}"
         echo ""
         offset_clock "${MODULE_STATUS_TIMER[${modindex}]}"
-        add_vote_table "${MODULE_STATUS[${modindex}]}" "${testtype}" "${MODULE_STATUS_MSG[${modindex}]}"
         if [[ ${MODULE_STATUS[${modindex}]} == -1
           && -n "${MODULE_STATUS_LOG[${modindex}]}" ]]; then
-          add_footer_table "${testtype}" "@@BASE@@/${MODULE_STATUS_LOG[${modindex}]}"
+          add_vote_table_v2 \
+            "${MODULE_STATUS[${modindex}]}" \
+            "${testtype}" \
+            "@@BASE@@/${MODULE_STATUS_LOG[${modindex}]}" \
+            "${MODULE_STATUS_MSG[${modindex}]}"
+          bugsystem_linecomments_queue \
+            "${testtype}" \
+            "${PATCH_DIR}/${MODULE_STATUS_LOG[${modindex}]}"
+        else
+          add_vote_table_v2 \
+            "${MODULE_STATUS[${modindex}]}" \
+            "${testtype}" \
+            "" \
+            "${MODULE_STATUS_MSG[${modindex}]}"
         fi
       fi
       ((modindex=modindex+1))
@@ -1826,7 +1812,7 @@ function modules_messages
     if [[ ${failure} == false ]]; then
       start_clock
       offset_clock "${goodtime}"
-      add_vote_table +1 "${testtype}" "${repo} passed${statusjdk}"
+      add_vote_table_v2 +1 "${testtype}" "" "${repo} passed${statusjdk}"
     fi
   else
     until [[ ${modindex} -eq ${#MODULE[@]} ]]; do
@@ -1835,10 +1821,19 @@ function modules_messages
       echo "${MODULE_STATUS_MSG[${modindex}]}"
       echo ""
       offset_clock "${MODULE_STATUS_TIMER[${modindex}]}"
-      add_vote_table "${MODULE_STATUS[${modindex}]}" "${testtype}" "${MODULE_STATUS_MSG[${modindex}]}"
       if [[ ${MODULE_STATUS[${modindex}]} == -1
         && -n "${MODULE_STATUS_LOG[${modindex}]}" ]]; then
-        add_footer_table "${testtype}" "@@BASE@@/${MODULE_STATUS_LOG[${modindex}]}"
+        add_vote_table_v2 \
+          "${MODULE_STATUS[${modindex}]}" \
+          "${testtype}" \
+          "@@BASE@@/${MODULE_STATUS_LOG[${modindex}]}" \
+          "${MODULE_STATUS_MSG[${modindex}]}"
+      else
+        add_vote_table_v2 \
+          "${MODULE_STATUS[${modindex}]}" \
+          "${testtype}" \
+          "" \
+          "${MODULE_STATUS_MSG[${modindex}]}"
       fi
       ((modindex=modindex+1))
     done
@@ -1964,13 +1959,13 @@ function modules_workers
 
     if [[ ${execvalue} == 0 ]] ; then
       module_status \
-        ${modindex} \
+        "${modindex}" \
         +1 \
         "${repostatus}-${testtype}-${fn}.txt" \
         "${modulesuffix} in ${repo} passed${statusjdk}."
     else
       module_status \
-        ${modindex} \
+        "${modindex}" \
         -1 \
         "${repostatus}-${testtype}-${fn}.txt" \
         "${modulesuffix} in ${repo} failed${statusjdk}."
@@ -2120,7 +2115,7 @@ function check_unittests
       jdk=${jdk// /}
     fi
 
-    personality_modules patch unit
+    personality_modules_wrapper patch unit
     "${BUILDTOOL}_modules_worker" patch unit
 
     ((result=result+$?))
@@ -2180,141 +2175,6 @@ function check_unittests
   return 0
 }
 
-## @description  Queue up comments to write into bug systems
-## @description  that have code review support, if such support
-## @description  enabled/available.
-## @description  File should be in the form of "file:line:comment"
-## @audience     public
-## @stability    evolving
-## @replaceable  no
-## @param        plugin
-## @param        filename
-function bugsystem_linecomments_queue
-{
-  declare plugin=$1
-  declare fn=$2
-  declare line
-  declare linenum
-  declare text
-
-  if [[ ! -f "${GITUNIDIFFLINES}" ]]; then
-    return
-  fi
-
-  while read -r line; do
-    file=$(echo "${line}" | cut -f1 -d:)
-    if [[ "${file}" =~ ^\./ ]]; then
-      file=${file:2}
-    fi
-    linenum=$(echo "${line}" | cut -f2 -d:)
-    text=$(echo "${line}" | cut -f3- -d:)
-
-    echo "${file}:${linenum}:${plugin}:${text}" >> "${PATCH_DIR}/linecomments-in.txt"
-
-  done < "${fn}"
-}
-
-## @description  Call relevant bug systems to write comments
-## @audience     private
-## @stability    evolving
-## @replaceable  no
-function bugsystem_linecomments_writer
-{
-  declare fn=$1
-  declare linenum=$2
-  shift 2
-  declare -a text
-  text=("$@")
-  declare idxline
-  declare uniline
-
-  if [[ ! -f "${GITUNIDIFFLINES}" ]]; then
-    return
-  fi
-
-  if [[ -z "${fn}" ]]; then
-    return
-  fi
-
-  idxline="${fn}:${linenum}:"
-  uniline=$("${GREP}" "${idxline}" "${GITUNIDIFFLINES}" | cut -f3 -d: )
-
-  for bugs in ${BUGLINECOMMENTS}; do
-    if declare -f "${bugs}_linecomments" >/dev/null;then
-      "${bugs}_linecomments" "${fn}" "${linenum}" "${uniline}" "${linetext[@]}"
-    fi
-  done
-}
-
-## @description  Write all of the bugsystem linecomments
-## @audience     public
-## @stability    evolving
-## @replaceable  no
-function bugsystem_linecomments_trigger
-{
-  declare line
-  declare fn
-  declare linenum
-  declare text
-  declare -a linetext
-  declare prevnum
-  declare prevfn
-  declare plugin
-
-  if [[ ! -f "${GITUNIDIFFLINES}" ]]; then
-    return 0
-  fi
-
-  if [[ ! -f "${PATCH_DIR}/linecomments-in.txt" ]]; then
-    return 0
-  fi
-
-  # sort the file such that all files and lines are now next to each other
-  sort "${PATCH_DIR}/linecomments-in.txt" > "${PATCH_DIR}/linecomments-sorted.txt"
-
-  while read -r line;do
-    fn=$(echo "${line}" | cut -f1 -d:)
-    if [[ -z "${prevfn}" ]]; then
-      prevfn=${fn}
-    fi
-
-    linenum=$(echo "${line}" | cut -f2 -d:)
-    if [[ -z "${prevnum}" ]]; then
-      prevnum=${linenum}
-    fi
-
-    text=$(echo "${line}" | cut -f3- -d:)
-
-    # if, for some reason either one of these
-    # isn't a number, force it to be zero.
-    if [[ ! ${prevnum} =~ ^[0-9]+$ ]]; then
-      prevnum=0
-    fi
-
-    if [[ ! ${linenum} =~ ^[0-9]+$ ]]; then
-      linenum=0
-    fi
-
-    if [[ "${prevfn}" == "${fn}" ]] &&
-       [[ ${prevnum} -eq ${linenum} ]]; then
-      linetext+=("${text}")
-      continue
-    fi
-
-    bugsystem_linecomments_writer "${prevfn}" "${prevnum}" "${linetext[@]}"
-    prevfn=${fn}
-    prevnum=${linenum}
-    linetext=("${text}")
-  done < "${PATCH_DIR}/linecomments-sorted.txt"
-
-  bugsystem_linecomments_writer "${prevfn}" "${prevnum}" "${linetext[@]}"
-  if [[ "${YETUS_SHELL_SCRIPT_DEBUG}" = true ]]; then
-    yetus_debug "Keeping linecomments files for debugging"
-  else
-    rm "${PATCH_DIR}/linecomments-in.txt" "${PATCH_DIR}/linecomments-sorted.txt"
-  fi
-}
-
 ## @description  Write the final output to the selected bug system
 ## @audience     private
 ## @stability    evolving
@@ -2336,6 +2196,8 @@ function bugsystem_finalreport
 
   add_footer_table "Powered by" "Apache Yetus ${VERSION} https://yetus.apache.org"
 
+  big_console_header "Generating Reports . . ."
+
   bugsystem_linecomments_trigger
 
   for bugs in ${BUGCOMMENTS}; do
@@ -2378,7 +2240,7 @@ function cleanup_and_exit
     big_console_header "Finished build."
   fi
 
-  if [[ "${DOCKERMODE}" != true ]]; then
+  if [[ "${DOCKERMODE}" != true  && -f "${PATCH_DIR}/pidfile.txt" ]]; then
     rm "${PATCH_DIR}/pidfile.txt"
   fi
 
@@ -2417,7 +2279,7 @@ function runtests
 
 ## @description  Calculate the differences between the specified files
 ## @description  using just the column+ messages (third+ column in a
-## @descriptoin  colon delimated flie) and output it to stdout.
+## @description  colon delimated file) and output it to stdout.
 ## @audience     public
 ## @stability    evolving
 ## @replaceable  no
@@ -2462,7 +2324,7 @@ function column_calcdiffs
 
 ## @description  Calculate the differences between the specified files
 ## @description  using just the error messages (last column in a
-## @descriptoin  colon delimated flie) and output it to stdout.
+## @description  colon delimated file) and output it to stdout.
 ## @audience     public
 ## @stability    evolving
 ## @replaceable  no
@@ -2539,7 +2401,7 @@ function calcdiffs
   fi
 }
 
-## @description generate a standarized calcdiff status message
+## @description generate a standardized calcdiff status message
 ## @audience    public
 ## @stability   evolving
 ## @replaceable no
@@ -2595,7 +2457,7 @@ function generic_logfilter
   fi
 }
 
-## @description  Helper routine for plugins to do a pre-patch prun
+## @description  Deprecated. Use module_pre_handler instead.
 ## @audience     public
 ## @stability    evolving
 ## @replaceable  no
@@ -2604,6 +2466,19 @@ function generic_logfilter
 ## @return       1 on failure
 ## @return       0 on success
 function generic_pre_handler
+{
+  module_pre_handler "$@"
+}
+
+## @description  Helper routine for plugins to do a pre-patch run
+## @audience     public
+## @stability    evolving
+## @replaceable  no
+## @param        testype
+## @param        multijdk
+## @return       1 on failure
+## @return       0 on success
+function module_pre_handler
 {
   declare testtype=$1
   declare multijdkmode=$2
@@ -2632,7 +2507,7 @@ function generic_pre_handler
       JAVA_HOME=${jdkindex}
     fi
 
-    personality_modules branch "${testtype}"
+    personality_modules_wrapper branch "${testtype}"
     "${BUILDTOOL}_modules_worker" branch "${testtype}"
 
     ((result=result + $?))
@@ -2647,7 +2522,15 @@ function generic_pre_handler
   return 0
 }
 
-## @description  Generic post-patch log handler
+## @description  Deprecated. Use module_postlog_compare instead.
+## @audience     public
+## @stability    evolving
+function generic_postlog_compare
+{
+  module_postlog_compare "$@"
+}
+
+## @description  Module post-patch log handler
 ## @audience     public
 ## @stability    evolving
 ## @replaceable  no
@@ -2656,7 +2539,7 @@ function generic_pre_handler
 ## @param        origlog
 ## @param        testtype
 ## @param        multijdkmode
-function generic_postlog_compare
+function module_postlog_compare
 {
   declare origlog=$1
   declare testtype=$2
@@ -2672,6 +2555,8 @@ function generic_postlog_compare
   declare -i samepatch=0
   declare -i fixedpatch=0
   declare summarize=true
+  declare tmpvar
+
 
   if [[ ${multijdk} == true ]]; then
     jdk=$(report_jvm_version "${JAVA_HOME}")
@@ -2709,19 +2594,19 @@ function generic_postlog_compare
     generic_logfilter "${testtype}" "${PATCH_DIR}/branch-${origlog}-${fn}.txt" "${PATCH_DIR}/branch-${origlog}-${testtype}-${fn}.txt"
     generic_logfilter "${testtype}" "${PATCH_DIR}/patch-${origlog}-${fn}.txt" "${PATCH_DIR}/patch-${origlog}-${testtype}-${fn}.txt"
 
-    # shellcheck disable=SC2016
-    numbranch=$(wc -l "${PATCH_DIR}/branch-${origlog}-${testtype}-${fn}.txt" | "${AWK}" '{print $1}')
-    # shellcheck disable=SC2016
-    numpatch=$(wc -l "${PATCH_DIR}/patch-${origlog}-${testtype}-${fn}.txt" | "${AWK}" '{print $1}')
+    tmpvar=$(wc -l "${PATCH_DIR}/branch-${origlog}-${testtype}-${fn}.txt")
+    numbranch=${tmpvar%% *}
+    tmpvar=$(wc -l "${PATCH_DIR}/patch-${origlog}-${testtype}-${fn}.txt")
+    numpatch=${tmpvar%% *}
 
     calcdiffs \
       "${PATCH_DIR}/branch-${origlog}-${testtype}-${fn}.txt" \
       "${PATCH_DIR}/patch-${origlog}-${testtype}-${fn}.txt" \
       "${testtype}" \
-      > "${PATCH_DIR}/diff-${origlog}-${testtype}-${fn}.txt"
+      > "${PATCH_DIR}/results-${origlog}-${testtype}-${fn}.txt"
 
-    # shellcheck disable=SC2016
-    addpatch=$(wc -l "${PATCH_DIR}/diff-${origlog}-${testtype}-${fn}.txt" | "${AWK}" '{print $1}')
+    tmpvar=$(wc -l "${PATCH_DIR}/results-${origlog}-${testtype}-${fn}.txt")
+    addpatch=${tmpvar%% *}
 
     ((fixedpatch=numbranch-numpatch+addpatch))
 
@@ -2729,7 +2614,7 @@ function generic_postlog_compare
 
     if [[ ${addpatch} -gt 0 ]]; then
       ((result = result + 1))
-      module_status "${i}" -1 "diff-${origlog}-${testtype}-${fn}.txt" "${fn}${statusjdk} ${statstring}"
+      module_status "${i}" -1 "results-${origlog}-${testtype}-${fn}.txt" "${fn}${statusjdk} ${statstring}"
     elif [[ ${fixedpatch} -gt 0 ]]; then
       module_status "${i}" +1 "${MODULE_STATUS_LOG[${i}]}" "${fn}${statusjdk} ${statstring}"
       summarize=false
@@ -2744,6 +2629,85 @@ function generic_postlog_compare
   return 0
 }
 
+## @description  Root-level post-patch log handler. Files should be
+## @description  linecomments compatible!
+## @audience     public
+## @stability    evolving
+## @replaceable  no
+## @return       0 on success
+## @return       1 on failure
+## @param        testtype
+## @param        branchlog
+## @param        patchlog
+function root_postlog_compare
+{
+  declare testtype=$1
+  declare branchlog=$2
+  declare patchlog=$3
+  declare fn
+  declare -i numbranch=0
+  declare -i numpatch=0
+  declare -i addpatch=0
+  declare -i samepatch=0
+  declare -i fixedpatch=0
+  declare tmpvar
+
+  yetus_debug "${testtype}: ${branchlog} vs. ${patchlog}"
+
+  # if it was a new module, this won't exist.
+  if [[ ! -f "${branchlog}" ]]; then
+    touch "${branchlog}"
+  fi
+
+  if [[ ! -f "${patchlog}" ]]; then
+    touch "${patchlog}"
+  fi
+
+  difflog="results-${testtype}.txt"
+
+  tmpvar=$(wc -l "${branchlog}")
+  numbranch="${tmpvar%% *}"
+  tmpvar=$(wc -l "${patchlog}")
+  numpatch="${tmpvar%% *}"
+
+  calcdiffs \
+    "${branchlog}" \
+    "${patchlog}" \
+    "${testtype}" \
+    > "${PATCH_DIR}/${difflog}"
+
+  tmpvar=$(wc -l "${PATCH_DIR}/${difflog}")
+  addpatch="${tmpvar%% *}"
+
+  ((fixedpatch=numbranch-numpatch+addpatch))
+
+  statstring=$(generic_calcdiff_status "${numbranch}" "${numpatch}" "${addpatch}" )
+
+  if [[ ${addpatch} -gt 0 ]]; then
+    add_vote_table_v2 -1 "${testtype}" "@@BASE@@/${difflog}" "${BUILDMODEMSG} ${statstring}"
+    bugsystem_linecomments_queue "${testtype}" "${PATCH_DIR}/${difflog}"
+    return 1
+  elif [[ ${fixedpatch} -gt 0 ]]; then
+   add_vote_table_v2 +1 "${testtype}" "" "${BUILDMODEMSG} ${statstring}"
+   return 0
+  fi
+
+  if [[ "${BUILDMODE}" == "full" ]]; then
+    add_vote_table_v2 +1 "${testtype}" "" "No issues."
+  else
+    add_vote_table_v2 +1 "${testtype}" "" "No new issues."
+  fi
+  return 0
+}
+
+## @description  Deprecated. Use module_post_handler instead.
+## @audience     public
+## @stability    evolving
+function generic_post_handler
+{
+  module_post_handler "$@"
+}
+
 ## @description  Generic post-patch handler
 ## @audience     public
 ## @stability    evolving
@@ -2754,7 +2718,7 @@ function generic_postlog_compare
 ## @param        testtype
 ## @param        multijdkmode
 ## @param        run commands
-function generic_post_handler
+function module_post_handler
 {
   declare origlog=$1
   declare testtype=$2
@@ -2784,7 +2748,7 @@ function generic_post_handler
     fi
 
     if [[ ${need2run} = true ]]; then
-      personality_modules "${codebase}" "${testtype}"
+      personality_modules_wrapper "${codebase}" "${testtype}"
       "${BUILDTOOL}_modules_worker" "${codebase}" "${testtype}"
 
       if [[ ${UNSUPPORTED_TEST} = true ]]; then
@@ -2792,7 +2756,7 @@ function generic_post_handler
       fi
     fi
 
-    generic_postlog_compare "${origlog}" "${testtype}" "${multijdkmode}"
+    module_postlog_compare "${origlog}" "${testtype}" "${multijdkmode}"
     ((result=result+$?))
   done
   JAVA_HOME=${savejavahome}
@@ -2860,7 +2824,7 @@ function compile_nonjvm
   declare multijdkmode=${2:-false}
   declare jdkindex=0
 
-  personality_modules "${codebase}" compile
+  personality_modules_wrapper "${codebase}" compile
   "${BUILDTOOL}_modules_worker" "${codebase}" compile
   modules_messages "${codebase}" compile true
 
@@ -3019,7 +2983,7 @@ function distclean
     fi
   done
 
-  personality_modules branch distclean
+  personality_modules_wrapper branch distclean
   "${BUILDTOOL}_modules_worker" branch distclean
   (( result = result + $? ))
 
@@ -3045,8 +3009,7 @@ function start_coprocessors
   if [[ "${BASH_VERSINFO[0]}" -gt 3 ]]; then
 
     for filename in "${BINDIR}/coprocs.d"/*; do
-      # shellcheck disable=SC1091
-      # shellcheck source=coprocs.d/process_counter.sh
+      # shellcheck source=SCRIPTDIR/coprocs.d/process_counter.sh
       . "${filename}"
     done
 
@@ -3096,7 +3059,7 @@ function patch_setup_work
   if ! dryrun_both_files; then
       ((RESULT = RESULT + 1))
       yetus_error "ERROR: ${PATCH_OR_ISSUE} does not apply to ${PATCH_BRANCH}."
-      add_vote_table -1 patch "${PATCH_OR_ISSUE} does not apply to ${PATCH_BRANCH}. Rebase required? Wrong Branch? See ${PATCH_NAMING_RULE} for help."
+      add_vote_table_v2 -1 patch "" "${PATCH_OR_ISSUE} does not apply to ${PATCH_BRANCH}. Rebase required? Wrong Branch? See ${PATCH_NAMING_RULE} for help."
       bugsystem_finalreport 1
       cleanup_and_exit 1
   fi
@@ -3123,6 +3086,8 @@ function initialize
 
   parse_args "$@"
 
+  check_basedir_repo
+
   importplugins
 
   if [[ -z "${BUILDTOOL}" ]]; then
@@ -3190,9 +3155,11 @@ function initialize
   if [[ "${#PARAMETER_TRACKER}" -gt 0 ]]; then
     yetus_error "ERROR: Unprocessed flag(s): ${PARAMETER_TRACKER[*]}"
     if [[ "${IGNORE_UNKNOWN_OPTIONS}" == true ]]; then
-      add_vote_table "-0" yetus "Unprocessed flag(s): ${PARAMETER_TRACKER[*]}"
+      if [[ "${REPORT_UNKNOWN_OPTIONS}" == true ]]; then
+        add_vote_table_v2 "-0" yetus "" "Unprocessed flag(s): ${PARAMETER_TRACKER[*]}"
+      fi
     else
-      add_vote_table -1 yetus "Unprocessed flag(s): ${PARAMETER_TRACKER[*]}"
+      add_vote_table_v2 -1 yetus "" "Unprocessed flag(s): ${PARAMETER_TRACKER[*]}"
       bugsystem_finalreport 1
       cleanup_and_exit 1
     fi
@@ -3263,9 +3230,8 @@ function import_core
   declare filename
 
   for filename in "${BINDIR}/core.d"/*; do
-    # shellcheck disable=SC1091
-    # shellcheck source=core.d/00-yetuslib.sh
-    # shellcheck source=core.d/01-common.sh
+    # shellcheck source=SCRIPTDIR/core.d/00-yetuslib.sh
+    # shellcheck source=SCRIPTDIR/core.d/01-common.sh
     . "${filename}"
   done
 }
@@ -3295,6 +3261,8 @@ setup_parameter_tracker
 
 import_core
 
+yetus_set_trap_handler generic_signal_handler HUP INT QUIT TERM
+
 if [[ "${BINNAME}" =~ qbt ]]; then
   initialize --empty-patch "$@"
 else
@@ -3318,7 +3286,7 @@ else
   yetus_debug "Changed process/Java native thread limit to ${PROC_LIMIT}"
 fi
 
-add_vote_table H "Prechecks"
+add_vote_table_v2 H "Prechecks"
 
 prechecks
 
@@ -3326,7 +3294,7 @@ if [[ "${BUILDMODE}" = patch ]]; then
 
   patchfiletests
 
-  add_vote_table H "${PATCH_BRANCH} Compile Tests"
+  add_vote_table_v2 H "${PATCH_BRANCH} Compile Tests"
 
   compile_cycle branch
 
@@ -3334,21 +3302,23 @@ if [[ "${BUILDMODE}" = patch ]]; then
 
   apply_patch_file
 
+  # Need to re-analyze the what files are changed due to
+  # potentially new exclusion rules
   exclude_paths_from_changed_files
 
   compute_gitdiff
 
-  add_vote_table H "Patch Compile Tests"
+  add_vote_table_v2 H "Patch Compile Tests"
 
 else
 
-  add_vote_table H "Compile Tests"
+  add_vote_table_v2 H "Compile Tests"
 
 fi
 
 compile_cycle patch
 
-add_vote_table H "Other Tests"
+add_vote_table_v2 H "Other Tests"
 
 runtests
 
@@ -3360,5 +3330,5 @@ finish_vote_table
 
 finish_footer_table
 
-bugsystem_finalreport ${RESULT}
-cleanup_and_exit ${RESULT}
+bugsystem_finalreport "${RESULT}"
+cleanup_and_exit "${RESULT}"
